@@ -21,10 +21,14 @@ module Test.Hspec.Core.Runner (
 , Path
 , defaultConfig
 , configAddFilter
+
+#ifdef TEST
+, rerunAll
+#endif
 ) where
 
 import           Prelude ()
-import           Test.Hspec.Compat
+import           Test.Hspec.Core.Compat
 
 import           Control.Monad
 import           Data.Maybe
@@ -40,10 +44,11 @@ import           Control.Monad.IO.Class (liftIO)
 
 import           Test.Hspec.Core.Util (Path)
 import           Test.Hspec.Core.Spec
-import           Test.Hspec.Config
+import           Test.Hspec.Core.Config
 import           Test.Hspec.Core.Formatters
 import           Test.Hspec.Core.Formatters.Internal
-import           Test.Hspec.FailureReport
+import qualified Test.Hspec.Core.Formatters.Internal as Formatter
+import           Test.Hspec.Core.FailureReport
 import           Test.Hspec.Core.QuickCheckUtil
 
 import           Test.Hspec.Core.Runner.Eval
@@ -78,7 +83,7 @@ applyDryRun c
   | otherwise = id
   where
     markSuccess :: Item () -> Item ()
-    markSuccess item = item {itemExample = evaluateExample Success}
+    markSuccess item = item {itemExample = safeEvaluateExample Success}
 
     removeCleanup :: SpecTree () -> SpecTree ()
     removeCleanup spec = case spec of
@@ -105,7 +110,10 @@ ensureSeed c = case configQuickCheckSeed c of
 hspecWith :: Config -> Spec -> IO ()
 hspecWith conf spec = do
   r <- hspecWithResult conf spec
-  unless (summaryFailures r == 0) exitFailure
+  unless (isSuccess r) exitFailure
+
+isSuccess :: Summary -> Bool
+isSuccess summary = summaryFailures summary == 0
 
 -- | Run given spec and returns a summary of the test run.
 --
@@ -121,57 +129,93 @@ hspecResult = hspecWithResult defaultConfig
 -- items.  If you need this, you have to check the `Summary` yourself and act
 -- accordingly.
 hspecWithResult :: Config -> Spec -> IO Summary
-hspecWithResult conf spec = do
+hspecWithResult config spec = do
   prog <- getProgName
   args <- getArgs
-  c <- getConfig conf prog args >>= ensureSeed
-  withArgs [] {- do not leak command-line arguments to examples -} $ withHandle c $ \h -> do
-    let formatter = fromMaybe specdoc (configFormatter c)
-        seed = (fromJust . configQuickCheckSeed) c
-        qcArgs = configQuickCheckArgs c
+  (oldFailureReport, c_) <- getConfig config prog args
+  c <- ensureSeed c_
+  if configRerunAllOnSuccess c
+    -- With --rerun-all we may run the spec twice. For that reason GHC can not
+    -- optimize away the spec tree. That means that the whole spec tree has to
+    -- be constructed in memory and we loose constant space behavior.
+    --
+    -- By separating between rerunAllMode and normalMode here, we retain
+    -- constant space behavior in normalMode.
+    --
+    -- see: https://github.com/hspec/hspec/issues/169
+    then rerunAllMode c oldFailureReport
+    else normalMode c
+  where
+    normalMode c = runSpec c spec
+    rerunAllMode c oldFailureReport = do
+      summary <- runSpec c spec
+      if rerunAll c oldFailureReport summary
+        then hspecWithResult config spec
+        else return summary
 
-    jobsSem <- newQSem =<< case configConcurrentJobs c of
+runSpec :: Config -> Spec -> IO Summary
+runSpec config spec = do
+  doNotLeakCommandLineArgumentsToExamples $ withHandle config $ \h -> do
+    let formatter = fromMaybe specdoc (configFormatter config)
+        seed = (fromJust . configQuickCheckSeed) config
+        qcArgs = configQuickCheckArgs config
+
+    jobsSem <- newQSem =<< case configConcurrentJobs config of
       Nothing -> getDefaultConcurrentJobs
       Just maxJobs -> return maxJobs
 
-    useColor <- doesUseColor h c
+    useColor <- doesUseColor h config
 
-    filteredSpec <- filterSpecs c . applyDryRun c <$> runSpecM spec
+    filteredSpec <- filterSpecs config . applyDryRun config <$> runSpecM spec
 
     withHiddenCursor useColor h $
-      runFormatM useColor (configHtmlOutput c) (configPrintCpuTime c) seed h $ do
-        runFormatter jobsSem useColor h c formatter filteredSpec `finally_` do
-          failedFormatter formatter
+      runFormatM useColor (configDiff config) (configHtmlOutput config) (configPrintCpuTime config) seed h $ do
+        runFormatter jobsSem useColor h config formatter filteredSpec `finally_` do
+          Formatter.interpret $ failedFormatter formatter
 
-        footerFormatter formatter
+        Formatter.interpret $ footerFormatter formatter
 
-        -- dump failure report
-        xs <- map failureRecordPath <$> getFailMessages
-        liftIO $ writeFailureReport FailureReport {
-            failureReportSeed = seed
-          , failureReportMaxSuccess = QC.maxSuccess qcArgs
-          , failureReportMaxSize = QC.maxSize qcArgs
-          , failureReportMaxDiscardRatio = QC.maxDiscardRatio qcArgs
-          , failureReportPaths = xs
-          }
+        xs <- map failureRecordPath <$> Formatter.interpret getFailMessages
+        liftIO $ dumpFailureReport config seed qcArgs xs
 
-        Summary <$> getTotalCount <*> getFailCount
-  where
-    withHiddenCursor :: Bool -> Handle -> IO a -> IO a
-    withHiddenCursor useColor h
-      | useColor  = E.bracket_ (hHideCursor h) (hShowCursor h)
-      | otherwise = id
+        Summary <$> Formatter.interpret getTotalCount <*> Formatter.interpret getFailCount
 
-    doesUseColor :: Handle -> Config -> IO Bool
-    doesUseColor h c = case configColorMode c of
-      ColorAuto  -> (&&) <$> hIsTerminalDevice h <*> (not <$> isDumb)
-      ColorNever -> return False
-      ColorAlways -> return True
+dumpFailureReport :: Config -> Integer -> QC.Args -> [Path] -> IO ()
+dumpFailureReport config seed qcArgs xs = do
+  writeFailureReport config FailureReport {
+      failureReportSeed = seed
+    , failureReportMaxSuccess = QC.maxSuccess qcArgs
+    , failureReportMaxSize = QC.maxSize qcArgs
+    , failureReportMaxDiscardRatio = QC.maxDiscardRatio qcArgs
+    , failureReportPaths = xs
+    }
 
-    withHandle :: Config -> (Handle -> IO a) -> IO a
-    withHandle c action = case configOutputFile c of
-      Left h -> action h
-      Right path -> withFile path WriteMode action
+doNotLeakCommandLineArgumentsToExamples :: IO a -> IO a
+doNotLeakCommandLineArgumentsToExamples = withArgs []
+
+withHiddenCursor :: Bool -> Handle -> IO a -> IO a
+withHiddenCursor useColor h
+  | useColor  = E.bracket_ (hHideCursor h) (hShowCursor h)
+  | otherwise = id
+
+doesUseColor :: Handle -> Config -> IO Bool
+doesUseColor h c = case configColorMode c of
+  ColorAuto  -> (&&) <$> hIsTerminalDevice h <*> (not <$> isDumb)
+  ColorNever -> return False
+  ColorAlways -> return True
+
+withHandle :: Config -> (Handle -> IO a) -> IO a
+withHandle c action = case configOutputFile c of
+  Left h -> action h
+  Right path -> withFile path WriteMode action
+
+rerunAll :: Config -> Maybe FailureReport -> Summary -> Bool
+rerunAll _ Nothing _ = False
+rerunAll config (Just oldFailureReport) summary =
+     configRerunAllOnSuccess config
+  && configRerun config
+  && isSuccess summary
+  && (not . null) (failureReportPaths oldFailureReport)
 
 isDumb :: IO Bool
 isDumb = maybe False (== "dumb") <$> lookupEnv "TERM"
