@@ -8,7 +8,7 @@
 module Test.Hspec.Core.Runner.Eval (runFormatter) where
 
 import           Prelude ()
-import           Test.Hspec.Compat
+import           Test.Hspec.Core.Compat
 
 import           Control.Monad (unless, when)
 import qualified Control.Exception as E
@@ -16,22 +16,22 @@ import           Control.Concurrent
 import           System.IO (Handle)
 
 import           Control.Monad.IO.Class (liftIO)
-import           Control.DeepSeq (deepseq)
 import           Data.Time.Clock.POSIX
 
 import           Test.Hspec.Core.Util
 import           Test.Hspec.Core.Spec
-import           Test.Hspec.Config
-import           Test.Hspec.Core.Formatters
+import           Test.Hspec.Core.Config
+import           Test.Hspec.Core.Formatters hiding (FormatM)
 import           Test.Hspec.Core.Formatters.Internal
-import           Test.Hspec.Timer
+import qualified Test.Hspec.Core.Formatters.Internal as Formatter
+import           Test.Hspec.Core.Timer
 
 type EvalTree = Tree (ActionWith ()) (String, Maybe Location, ProgressCallback -> FormatResult -> IO (FormatM ()))
 
 -- | Evaluate all examples of a given spec and produce a report.
 runFormatter :: QSem -> Bool -> Handle -> Config -> Formatter -> [SpecTree ()] -> FormatM ()
 runFormatter jobsSem useColor h c formatter specs = do
-  headerFormatter formatter
+  Formatter.interpret $ headerFormatter formatter
   chan <- liftIO newChan
   reportProgress <- liftIO mkReportProgress
   run chan reportProgress c formatter (toEvalTree specs)
@@ -60,24 +60,24 @@ every seconds action = do
 
 type FormatResult = Either E.SomeException Result -> FormatM ()
 
-parallelize :: QSem -> Bool -> (ProgressCallback -> IO Result) -> ProgressCallback -> FormatResult -> IO (FormatM ())
+parallelize :: QSem -> Bool -> (ProgressCallback -> IO (Either E.SomeException Result)) -> ProgressCallback -> FormatResult -> IO (FormatM ())
 parallelize jobsSem isParallelizable e
   | isParallelizable = runParallel jobsSem e
   | otherwise = runSequentially e
 
-runSequentially :: (ProgressCallback -> IO Result) -> ProgressCallback -> FormatResult -> IO (FormatM ())
+runSequentially :: (ProgressCallback -> IO (Either E.SomeException Result)) -> ProgressCallback -> FormatResult -> IO (FormatM ())
 runSequentially e reportProgress formatResult = return $ do
-  result <- liftIO $ evalExample (e reportProgress)
+  result <- liftIO $ e reportProgress
   formatResult result
 
 data Report = ReportProgress Progress | ReportResult (Either E.SomeException Result)
 
-runParallel :: QSem -> (ProgressCallback -> IO Result) -> ProgressCallback -> FormatResult -> IO (FormatM ())
+runParallel :: QSem -> (ProgressCallback -> IO (Either E.SomeException Result)) -> ProgressCallback -> FormatResult -> IO (FormatM ())
 runParallel jobsSem e reportProgress formatResult = do
   mvar <- newEmptyMVar
   _ <- forkIO $ E.bracket_ (waitQSem jobsSem) (signalQSem jobsSem) $ do
     let progressCallback = replaceMVar mvar . ReportProgress
-    result <- evalExample (e progressCallback)
+    result <- e progressCallback
     replaceMVar mvar (ReportResult result)
   return $ evalReport mvar
   where
@@ -93,15 +93,6 @@ runParallel jobsSem e reportProgress formatResult = do
     replaceMVar :: MVar a -> a -> IO ()
     replaceMVar mvar p = tryTakeMVar mvar >> putMVar mvar p
 
-evalExample :: IO Result -> IO (Either E.SomeException Result)
-evalExample e = safeTry $ forceResult <$> e
-  where
-    forceResult :: Result -> Result
-    forceResult r = case r of
-      Success -> r
-      Pending m -> m `deepseq` r
-      Fail _ m -> m `deepseq` r
-
 data Message = Done | Run (FormatM ())
 
 run :: Chan Message -> (Path -> ProgressCallback) -> Config -> Formatter -> [EvalTree] -> FormatM ()
@@ -121,9 +112,9 @@ run chan reportProgress_ c formatter specs = do
 
     queueSpec :: [String] -> EvalTree -> IO ()
     queueSpec rGroups (Node group xs) = do
-      defer (exampleGroupStarted formatter (reverse rGroups) group)
+      defer (Formatter.interpret $ exampleGroupStarted formatter (reverse rGroups) group)
       forM_ xs (queueSpec (group : rGroups))
-      defer (exampleGroupDone formatter)
+      defer (Formatter.interpret $ exampleGroupDone formatter)
     queueSpec rGroups (NodeWithCleanup action xs) = do
       forM_ xs (queueSpec rGroups)
       defer (runCleanup (action ()) (reverse rGroups, "afterAll-hook"))
@@ -138,22 +129,22 @@ run chan reportProgress_ c formatter specs = do
 
         reportProgress = reportProgress_ path
 
-        formatResult :: Either E.SomeException Result -> FormatM ()
+        formatResult :: FormatResult
         formatResult result = do
           case result of
             Right Success -> do
               increaseSuccessCount
-              exampleSucceeded formatter path
+              Formatter.interpret $ exampleSucceeded formatter path
             Right (Pending reason) -> do
               increasePendingCount
-              examplePending formatter path reason
-            Right (Fail loc_ err) -> failed (loc_ <|> loc) path (Right err)
+              Formatter.interpret $ examplePending formatter path reason
+            Right (Failure loc_ err) -> failed (loc_ <|> loc) path (Right err)
             Left err         -> failed loc path (Left  err)
 
     failed loc path err = do
       increaseFailCount
       addFailMessage loc path err
-      exampleFailed formatter path err
+      Formatter.interpret $ exampleFailed formatter path err
 
 processMessages :: IO Message -> Bool -> FormatM ()
 processMessages getMessage fastFail = go
@@ -161,6 +152,6 @@ processMessages getMessage fastFail = go
     go = liftIO getMessage >>= \m -> case m of
       Run action -> do
         action
-        fails <- getFailCount
+        fails <- Formatter.interpret getFailCount
         unless (fastFail && fails /= 0) go
       Done -> return ()
