@@ -1,8 +1,5 @@
-{-# LANGUAGE CPP #-}
-#if MIN_VERSION_base(4,8,1)
-#define HAS_SOURCE_LOCATIONS
-{-# LANGUAGE ImplicitParams #-}
-#endif
+{-# LANGUAGE FlexibleContexts #-}
+{-# LANGUAGE ConstraintKinds #-}
 -- |
 -- Stability: unstable
 --
@@ -11,10 +8,16 @@
 module Test.Hspec.Core.Spec (
 
 -- * Defining a spec
-  describe
-, it
+  it
+, specify
+, describe
+, context
 , pending
 , pendingWith
+, xit
+, xspecify
+, xdescribe
+, xcontext
 , parallel
 
 -- * The @SpecM@ monad
@@ -27,15 +30,13 @@ module Test.Hspec.Core.Spec (
 , module Test.Hspec.Core.Tree
 ) where
 
-#ifdef HAS_SOURCE_LOCATIONS
-import           GHC.Stack
-#endif
-
 import qualified Control.Exception as E
+import           Data.CallStack
 
 import           Test.Hspec.Expectations (Expectation)
 
 import           Test.Hspec.Core.Example
+import           Test.Hspec.Core.Hooks
 import           Test.Hspec.Core.Tree
 import           Test.Hspec.Core.Spec.Monad
 
@@ -43,6 +44,21 @@ import           Test.Hspec.Core.Spec.Monad
 describe :: String -> SpecWith a -> SpecWith a
 describe label spec = runIO (runSpecM spec) >>= fromSpecList . return . specGroup label
 
+-- | @context@ is an alias for `describe`.
+context :: String -> SpecWith a -> SpecWith a
+context = describe
+
+-- |
+-- Changing `describe` to `xdescribe` marks all spec items of the corresponding subtree as pending.
+--
+-- This can be used to temporarily disable spec items.
+xdescribe :: String -> SpecWith a -> SpecWith a
+xdescribe label spec = before_ pending $ describe label spec
+
+-- | @xcontext@ is an alias for `xdescribe`.
+xcontext :: String -> SpecWith a -> SpecWith a
+xcontext = xdescribe
+
 -- | The @it@ function creates a spec item.
 --
 -- A spec item consists of:
@@ -54,19 +70,30 @@ describe label spec = runIO (runSpecM spec) >>= fromSpecList . return . specGrou
 -- > describe "absolute" $ do
 -- >   it "returns a positive number when given a negative number" $
 -- >     absolute (-1) == 1
-#ifdef HAS_SOURCE_LOCATIONS
-it :: (?loc :: CallStack, Example a) => String -> a -> SpecWith (Arg a)
-#else
-it :: Example a => String -> a -> SpecWith (Arg a)
-#endif
+it :: (HasCallStack, Example a) => String -> a -> SpecWith (Arg a)
 it label action = fromSpecList [specItem label action]
 
+-- | @specify@ is an alias for `it`.
+specify :: (HasCallStack, Example a) => String -> a -> SpecWith (Arg a)
+specify = it
+
+-- |
+-- Changing `it` to `xit` marks the corresponding spec item as pending.
+--
+-- This can be used to temporarily disable a spec item.
+xit :: (HasCallStack, Example a) => String -> a -> SpecWith (Arg a)
+xit label action = before_ pending $ it label action
+
+-- | @xspecify@ is an alias for `xit`.
+xspecify :: (HasCallStack, Example a) => String -> a -> SpecWith (Arg a)
+xspecify = xit
+
 -- | `parallel` marks all spec items of the given spec to be safe for parallel
 -- evaluation.
 parallel :: SpecWith a -> SpecWith a
 parallel = mapSpecItem_ $ \item -> item {itemIsParallelizable = True}
 
--- | `pending` can be used to indicate that an example is /pending/.
+-- | `pending` can be used to mark a spec item as pending.
 --
 -- If you want to textually specify a behavior but do not have an example yet,
 -- use this:
@@ -79,6 +106,6 @@ pending = E.throwIO (Pending Nothing)
 
 -- |
 -- `pendingWith` is similar to `pending`, but it takes an additional string
--- argument that can be used to specify the reason for why it's pending.
+-- argument that can be used to specify the reason for why the spec item is pending.
 pendingWith :: String -> Expectation
 pendingWith = E.throwIO . Pending . Just
