@@ -1,10 +1,8 @@
 {-# LANGUAGE DeriveFunctor #-}
-
-{-# LANGUAGE CPP #-}
-#if MIN_VERSION_base(4,8,1)
-#define HAS_SOURCE_LOCATIONS
-{-# LANGUAGE ImplicitParams #-}
-#endif
+{-# LANGUAGE DeriveFoldable #-}
+{-# LANGUAGE DeriveTraversable #-}
+{-# LANGUAGE FlexibleContexts #-}
+{-# LANGUAGE ConstraintKinds #-}
 
 -- |
 -- Stability: unstable
@@ -16,15 +14,11 @@ module Test.Hspec.Core.Tree (
 , specItem
 ) where
 
-#ifdef HAS_SOURCE_LOCATIONS
-#if !MIN_VERSION_base(4,9,0)
-import           GHC.SrcLoc
-#endif
-import           GHC.Stack
-#endif
+import           Data.CallStack
+import           Control.Exception
 
 import           Prelude ()
-import           Test.Hspec.Compat
+import           Test.Hspec.Core.Compat
 
 import           Test.Hspec.Core.Example
 
@@ -33,25 +27,7 @@ data Tree c a =
     Node String [Tree c a]
   | NodeWithCleanup c [Tree c a]
   | Leaf a
-  deriving Functor
-
-instance Foldable (Tree c) where -- Note: GHC 7.0.1 fails to derive this instance
-  foldMap = go
-    where
-      go :: Monoid m => (a -> m) -> Tree c a -> m
-      go f t = case t of
-        Node _ xs -> foldMap (foldMap f) xs
-        NodeWithCleanup _ xs -> foldMap (foldMap f) xs
-        Leaf x -> f x
-
-instance Traversable (Tree c) where -- Note: GHC 7.0.1 fails to derive this instance
-  sequenceA = go
-    where
-      go :: Applicative f => Tree c (f a) -> f (Tree c a)
-      go t = case t of
-        Node label xs -> Node label <$> sequenceA (map go xs)
-        NodeWithCleanup action xs -> NodeWithCleanup action <$> sequenceA (map go xs)
-        Leaf a -> Leaf <$> a
+  deriving (Functor, Foldable, Traversable)
 
 -- | A tree is used to represent a spec internally.  The tree is parametrize
 -- over the type of cleanup actions and the type of the actual spec items.
@@ -76,7 +52,7 @@ data Item a = Item {
   -- parallel with other spec items
 , itemIsParallelizable :: Bool
   -- | Example for behavior
-, itemExample :: Params -> (ActionWith a -> IO ()) -> ProgressCallback -> IO Result
+, itemExample :: Params -> (ActionWith a -> IO ()) -> ProgressCallback -> IO (Either SomeException Result)
 }
 
 -- | The @specGroup@ function combines a list of specs into a larger spec.
@@ -88,22 +64,14 @@ specGroup s = Node msg
       | otherwise = s
 
 -- | The @specItem@ function creates a spec item.
-#ifdef HAS_SOURCE_LOCATIONS
-specItem :: (?loc :: CallStack, Example a) => String -> a -> SpecTree (Arg a)
-#else
-specItem :: Example a => String -> a -> SpecTree (Arg a)
-#endif
-specItem s e = Leaf $ Item requirement location False (evaluateExample e)
+specItem :: (HasCallStack, Example a) => String -> a -> SpecTree (Arg a)
+specItem s e = Leaf $ Item requirement location False (safeEvaluateExample e)
   where
     requirement
       | null s = "(unspecified behavior)"
       | otherwise = s
 
     location :: Maybe Location
-#ifdef HAS_SOURCE_LOCATIONS
-    location = case reverse (getCallStack ?loc) of
+    location = case reverse callStack of
       (_, loc) : _ -> Just (Location (srcLocFile loc) (srcLocStartLine loc) (srcLocStartCol loc) ExactLocation)
       _ -> Nothing
-#else
-    location = Nothing
-#endif
